<?php

namespace Drupal\user_account_language_negotiation;

use Drupal\Component\Gettext\PoItem;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManager;
use Drupal\locale\PoDatabaseWriter;
use Drupal\user_account_language_negotiation\Plugin\LanguageNegotiation\LanguageNegotiationUserAccountSaver;

/**
 * Class for hooks on installing and uninstalling the module.
 */
class ModuleInstallHandler {

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The config key for enabled language negotiation plugins.
   */
  private const CONFIG_KEY = 'negotiation.language_interface.enabled';

  /**
   * Constructs a ModuleInstallHandler object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Run this when the module is installed.
   */
  public function onInstall(): void {
    $this->translateLanguageNames();
  }

  /**
   * When the module is uninstalled.
   */
  public function onUninstall(): void {
    $this->disableOurPluginIfNeeded();
  }

  /**
   * Import translations for installed languages.
   */
  private function translateLanguageNames(): void {
    $writer = new PoDatabaseWriter();
    $writer->setOptions(['overwrite_options' => []]);
    $item = new PoItem();
    foreach (LanguageManager::getStandardLanguageList() as $langcode => $long_versions) {
      if ($long_versions[0] == $long_versions[1]) {
        // No translation needed.
        continue;
      }

      $item->setSource($long_versions[0]);
      $item->setTranslation($long_versions[1]);

      $writer->setLangcode($langcode);
      $writer->writeItem($item);
    }
  }

  /**
   * Prevent exception "The language-user-account-saver plugin does not exist".
   */
  private function disableOurPluginIfNeeded(): void {
    $config = $this->configFactory->getEditable('language.types');

    $enabled_plugins = $config->get(self::CONFIG_KEY);
    unset($enabled_plugins[LanguageNegotiationUserAccountSaver::METHOD_ID]);
    $config->set(self::CONFIG_KEY, $enabled_plugins);
    $config->save();
  }

}
